@@ -11,7 +11,6 @@ import hmac
 import base64
 import utils
 import re
-from xml.etree.ElementTree import fromstring
 try:
     from xml.etree.ElementTree import ParseError as XMLError
 except ImportError:
@@ -22,6 +21,32 @@ from requests import request
 from requests.exceptions import HTTPError
 
 
+__all__ = [
+    'Feeds',
+    'Inventory',
+    'Reports',
+    'Orders',
+    'Products',
+    'Recommendations',
+    'Sellers',
+]
+
+# See https://images-na.ssl-images-amazon.com/images/G/01/mwsportal/doc/en_US/bde/MWSDeveloperGuide._V357736853_.pdf page 8
+# for a list of the end points and marketplace IDs
+
+MARKETPLACES = {
+    "CA" : "https://mws.amazonservices.ca", #A2EUQ1WTGCTBG2
+    "US" : "https://mws.amazonservices.com", #ATVPDKIKX0DER",
+    "DE" : "https://mws-eu.amazonservices.com", #A1PA6795UKMFR9
+    "ES" : "https://mws-eu.amazonservices.com", #A1RKKUPIHCS9HS
+    "FR" : "https://mws-eu.amazonservices.com", #A13V1IB3VIYZZH
+    "IN" : "https://mws.amazonservices.in", #A21TJRUUN4KGV
+    "IT" : "https://mws-eu.amazonservices.com", #APJ6JRA9NG5V4
+    "UK" : "https://mws-eu.amazonservices.com", #A1F83G8C2ARO7P
+    "JP" : "https://mws.amazonservices.jp", #A1VC38T7YXB528
+    "CN" : "https://mws.amazonservices.com.cn", #AAHKV2X7AFYLW
+}
+
 
 class MWSError(Exception):
     """
@@ -115,14 +140,23 @@ class MWS(object):
     # Which is the name of the parameter for that specific account type.
     ACCOUNT_TYPE = "SellerId"
 
-    def __init__(self, access_key, secret_key, account_id,
-                 domain='https://mws.amazonservices.com', uri="", version=""):
+    def __init__(self, access_key, secret_key, account_id, region='US', domain='', uri="", version=""):
         self.access_key = access_key
         self.secret_key = secret_key
         self.account_id = account_id
-        self.domain = domain
-        self.uri = uri or self.URI
         self.version = version or self.VERSION
+        self.uri = uri or self.URI
+
+        if domain:
+            self.domain = domain
+        elif region in MARKETPLACES:
+            self.domain = MARKETPLACES[region]
+        else:
+            error_msg = "Incorrect region supplied ('%(region)s'). Must be one of the following: %(marketplaces)s" % {
+                "marketplaces" : ', '.join(MARKETPLACES.keys()),
+                "region" : region,
+            }
+            raise MWSError(error_msg)
 
     def make_request(self, extra_data, method="GET", **kwargs):
         """Make request to Amazon MWS API with these parameters
@@ -561,7 +595,39 @@ class OutboundShipments(MWS):
     # To be completed
 
 
-try:
-    execfile('keys.py', globals(), locals())
-except IOError, err:
-    pass
+class Recommendations(MWS):
+
+    """ Amazon MWS Recommendations API """
+
+    URI = '/Recommendations/2013-04-01'
+    VERSION = '2013-04-01'
+    NS = "{https://mws.amazonservices.com/Recommendations/2013-04-01}"
+
+    def get_last_updated_time_for_recommendations(self, marketplaceid):
+        """
+        Checks whether there are active recommendations for each category for the given marketplace, and if there are,
+        returns the time when recommendations were last updated for each category.
+        """
+
+        data = dict(Action='GetLastUpdatedTimeForRecommendations',
+                    MarketplaceId=marketplaceid)
+        return self.make_request(data, "POST")
+
+    def list_recommendations(self, marketplaceid, recommendationcategory=None):
+        """
+        Returns your active recommendations for a specific category or for all categories for a specific marketplace.
+        """
+
+        data = dict(Action="ListRecommendations",
+                    MarketplaceId=marketplaceid,
+                    RecommendationCategory=recommendationcategory)
+        return self.make_request(data, "POST")
+
+    def list_recommendations_by_next_token(self, token):
+        """
+        Returns the next page of recommendations using the NextToken parameter.
+        """
+
+        data = dict(Action="ListRecommendationsByNextToken",
+                    NextToken=token)
+        return self.make_request(data, "POST")
